"""
 Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 SPDX-License-Identifier: MIT-0

 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

"""
This class is compatible with Amazon Lex V2 data structure.
It allows to retrieve specific attributes of the object by means of getter methods
and to modify attributes by means of setters.
"""


class LexEvent:

    def __init__(self, request):
        self.req = request

    def is_input_user_request(self):
        return self.req.get("invocationSource") == "DialogCodeHook"

    def intent_is_fulfilled (self):
        return self.req.get("invocationSource") == "FulfillmentCodeHook"

    def get_input_mode (self):
        return self.req.get("inputMode")

    def get_input_transcript (self):
        return self.req.get("inputTranscript")

    def get_session_attr (self, attr):
        return self.req["sessionState"]["sessionAttributes"].get(attr)

    def set_session_attr(self, attr, attr_val):
        self.req["sessionState"]["sessionAttributes"][attr] = attr_val
        return self

    def get_session_attrs(self):
        satts = self.req["sessionState"].get("sessionAttributes")
        if not satts:
            self.req["sessionState"]["sessionAttributes"] = {}
            satts = self.req["sessionState"].get("sessionAttributes")
        return satts

    def get_current_intent_slot (self, slot_name):
        current_intent_slots = self.req['currentIntent']['slots']
        return current_intent_slots.get(slot_name)

    def get_interpretations(self):
        return self.req.get("interpretations")

    def get_confirmation_state (self):
        return self.req["sessionState"]["intent"].get("confirmationState")

    def is_confirmed(self):
        return self.get_confirmation_state() == "Confirmed"

    def get_slot(self, slot_name):
        return self.req['sessionState']['intent']["slots"].get(slot_name)

    def get_current_intent(self):
        return self.req['sessionState']['intent']

    def get_current_intent_name(self) -> str:
        return self.req['sessionState']['intent'].get('name')

    def slot_exists(self, slot_name):
        return self.get_slot(slot_name) is not None

    def get_interpreted_value (self, slot_name):
        slot = self.get_slot(slot_name)
        if slot:
            return slot['value'].get('interpretedValue')
        else:
            return None

    def increase_retry(self, slot_name: str):
        attrs = self.get_session_attrs()
        attr_retry = 'retries_{}'.format(slot_name.lower())
        if attr_retry in attrs:
            attrs[attr_retry] = int(attrs[attr_retry]) + 1
        else:
            attrs[attr_retry] = 1
        return attrs[attr_retry]

    def is_requesting_slot (self, slot_name: str):
        dialog_action = self.req.get("proposedNextState").get("dialogAction")
        if dialog_action:
            type = dialog_action.get("type")
            slot_to_elicit = dialog_action.get("slotToElicit")
            return type == "ElicitSlot" and slot_to_elicit.lower() == slot_name.lower()
        return False



import pkg_resources
import json
from unittest import TestCase
from amazon_lex_helper import IntentHandler
from amazon_lex_helper import LexEventDispatcher
from amazon_lex_helper import Disambiguation
from amazon_lex_helper import LexEvent

class LexEventDispatcherTests (TestCase):

    def test_duplicated_intent_raises_exception(self):
        a = IntentHandler("foo")
        b = IntentHandler("foo")
        event_dispatcher = LexEventDispatcher()
        self.assertRaises(AssertionError, event_dispatcher.subscribe, a, b)

    def test_different_intents_are_successfully_registered(self):
        a = IntentHandler("foo")
        b = IntentHandler("bar")
        event_dispatcher = LexEventDispatcher().subscribe(a, b)
        self.assertTrue(len(event_dispatcher.subscribers) == 2)

    def test_unknown_intents_are_delegated_to_lex(self):
        path = 'samples/trade.json'
        full_path = pkg_resources.resource_filename(__name__, path)
        with open(full_path) as json_file:
            raw_event = json.load(json_file)
            event_dispatcher = LexEventDispatcher()
            event_dispatcher.dispatch(raw_event)

    def test_use_ambiguity_handler_if_provided (self):
        path = 'samples/trade.json'
        full_path = pkg_resources.resource_filename(__name__, path)
        with open(full_path) as json_file:
            raw_event = json.load(json_file)
            a = IntentHandler("foo")
            b = IntentHandler("bar")
            event_dispatcher = LexEventDispatcher().subscribe(a, b)
            event_dispatcher.set_ambiguity_handler(Disambiguation(0.5))
            response = event_dispatcher.dispatch(raw_event)



"""
 Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 SPDX-License-Identifier: MIT-0

 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

"""
This module is compatible with the Amazon Lex V2 data structure. 
It provides methods to build custom responses returned by the Lambda function to Amazon Lex.

For instructions on how to set up and test this bot, as well as additional samples,
visit the Lex Getting Started documentation http://docs.aws.amazon.com/lex/latest/dg/getting-started.html.
"""
from amazon_lex_helper import LexEvent


def ask_due_to_ambiguity (req: LexEvent, intent_name, message=None):
    return elicit_intent(req, intent_name, "In Progress", message)


def elicit_intent(req: LexEvent, intent_name, state, message=None):
    resp = {
        "sessionState": {
            'activeContexts': [{
                'name': 'intentContext',
                'contextAttributes': {},
                'timeToLive': {'timeToLiveInSeconds': 600, 'turnsToLive': 1}
            }],
            'sessionAttributes': req.get_session_attrs(),
            'dialogAction': {'type': 'ElicitIntent'},
            'intent': {'name': intent_name, 'state': state},
        }
    }
    if message:
        resp['messages'] = [{'contentType': 'PlainText', 'content': message}]
    return resp


def elicit_slot(req: LexEvent, slot_to_elicit, message=None):
    resp = {
        'sessionState': {
            'activeContexts': [{
                'name': 'intentContext',
                'contextAttributes': {},
                'timeToLive': {'timeToLiveInSeconds': 600, 'turnsToLive': 1}
            }],
            'sessionAttributes': req.get_session_attrs() or {},
            'dialogAction': {
                'type': 'ElicitSlot',
                'slotToElicit': slot_to_elicit
            },
            'intent': req.get_current_intent()
        }
    }
    if message:
        resp['messages'] = [{'contentType': 'PlainText', 'content': message}]
    return resp


def confirm_intent(session_attributes, active_contexts, intent, message=None):
    resp = {
        'sessionState': {
            'activeContexts': [{
                'name': 'intentContext',
                'contextAttributes': active_contexts,
                'timeToLive': {
                    'timeToLiveInSeconds': 600,
                    'turnsToLive': 1
                }
            }],
            'sessionAttributes': session_attributes,
            'dialogAction': {
                'type': 'ConfirmIntent'
            },
            'intent': intent
        }
    }
    if message:
        resp['messages'] = [{'contentType': 'PlainText', 'content': message}]
    return resp


def close(session_attributes, intent, context_attrs, message=None):
    resp = {
        'sessionState': {
            'activeContexts': [{
                'name': 'intentContext',
                'contextAttributes': context_attrs,
                'timeToLive': {
                    'timeToLiveInSeconds': 600,
                    'turnsToLive': 1
                }
            }],
            'sessionAttributes': session_attributes,
            'dialogAction': {'type': 'Close'},
            'intent': intent
        }
    }
    if message:
        resp['messages'] = [{'contentType': 'PlainText', 'content': message}]
    return resp


def delegate (req: LexEvent, message=None):
    session_attributes = req.get_session_attrs() or {}
    context_attrs = {}
    intent = req.get_current_intent()
    resp = {
        'sessionState': {
            'activeContexts': [{
                'name': 'intentContext',
                'contextAttributes': context_attrs,
                'timeToLive': {
                    'timeToLiveInSeconds': 600,
                    'turnsToLive': 1
                }
            }],
            'sessionAttributes': session_attributes,
            'dialogAction': {'type': 'Delegate'},
            'intent': intent
        }
    }
    if message:
        resp['messages'] = [{'contentType': 'PlainText', 'content': message}]
    return resp


def initial_message(intent_name):
    slot_to_elicit = 'Location' if intent_name == 'BookHotel' else 'PickUpCity'
    return {
        'sessionState': {
            'dialogAction': {
                'type': 'ElicitSlot',
                'slotToElicit': slot_to_elicit
            },
            'intent': {
                'confirmationState': 'None',
                'name': intent_name,
                'state': 'InProgress'
            }
        }
    }



"""
 Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 SPDX-License-Identifier: MIT-0

 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

import logging
from abc import abstractmethod

from amazon_lex_helper import LexEvent

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


class IntentHandler:
    """
    Handles a Lex event received by the dispatcher.
    Each intent handler is subscribed to the dispatcher by intent name, so only one handler is allowed for each
    of the intents defined in the Lex bot.
    """
    def __init__(self, intent_name):
        self.intent_name = intent_name

    def get_intent_name(self):
        return self.intent_name

    @abstractmethod
    def process_request(self, request: LexEvent):
        pass

    def log(self):
        return logger

    def valid_intent(self, lex: LexEvent):
        valid = False
        slots = lex.get_session_slots()
        if slots:
            none_slots = [slot_name for slot_name in slots if not slots[slot_name]]
            logger.debug("intent list = {}".format(none_slots))
            valid = len(none_slots) == 0
        return valid


from unittest import TestCase
from amazon_lex_helper import LexEvent, LexResponse
import pkg_resources
import json


class LexResponseTests (TestCase):

    def test_delegate_returns_not_null (self):
        path = 'samples/trade.json'
        full_path = pkg_resources.resource_filename(__name__, path)
        with open(full_path) as json_file:
            json_data = json.load(json_file)
            req = LexEvent(json_data)
            resp = LexResponse.delegate(req)
            self.assertIsNotNone(resp)



import json
import pkg_resources
from unittest import TestCase

from amazon_lex_helper import LexEvent


class LexRequestTests (TestCase):

    def test_request_is_parsed (self):
        path = 'samples/trade.json'
        full_path = pkg_resources.resource_filename(__name__, path)
        with open(full_path) as json_file:
            json_data = json.load(json_file)
            req = LexEvent(json_data)
            self.assertEqual(req.get_current_intent()['name'], "OpenFuturesIntent")
            self.assertEqual(req.get_input_transcript(), "trade")

    def test_increase_retry(self):
        path = 'samples/trade.json'
        full_path = pkg_resources.resource_filename(__name__, path)
        with open(full_path) as json_file:
            json_data = json.load(json_file)
            req = LexEvent(json_data)
            req.increase_retry("test_slot")
            req.increase_retry("test_slot")
            self.assertEqual(req.get_session_attr("retries_test_slot"), 2)




import json
import unittest
import pkg_resources

from amazon_lex_helper import LexEvent
from amazon_lex_helper import Disambiguation


class DisambiguationTests(unittest.TestCase):

    def test_ambiguity_limit_below_threshold (self):
        path = 'samples/trade.json'
        full_path = pkg_resources.resource_filename(__name__, path)
        with open(full_path) as json_file:
            json_data = json.load(json_file)
            req = LexEvent(json_data)
            d = Disambiguation(0.2)
            ambiguity_data = d.check_ambiguity_limit(req)
            self.assertIsNone(ambiguity_data)

    def test_ambiguity_limit_over_threshold (self):
        path = 'samples/trade.json'
        full_path = pkg_resources.resource_filename(__name__, path)
        with open(full_path) as json_file:
            json_data = json.load(json_file)
            req = LexEvent(json_data)
            d = Disambiguation(0.5)
            ambiguity_data = d.check_ambiguity_limit(req)
            self.assertIsNotNone(ambiguity_data)


from amazon_lex_helper.ambiguity.Disambiguation import Disambiguation
from amazon_lex_helper.LexEvent import LexEvent
from amazon_lex_helper import LexResponse
from amazon_lex_helper.LexEventDispatcher import LexEventDispatcher
from amazon_lex_helper.IntentHandler import IntentHandler




from setuptools import setup, find_packages
import os

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="amazon-lex-helper",
    packages=['amazon_lex_helper'],
    version="0.5",
    author="rubenafo",
    author_email='rubenafo@amazon.com',
    description="Interact with Amazon Lex using Python in an easy way",
    url="https://github.com/aws-samples/amazon-lex-v2-helper",
    download_url = 'https://github.com/aws-samples/amazon-lex-v2-helper/amazon-lex-v2-helper-v0.1.tgz',
    keywords = ["amazon-lex", "lex", "chatbot", "voicebot", "nlp", "aws"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Utilities",
        "Development Status :: 4 - Beta",
    ],
    python_requires='>=3.6',
)



"""
 Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 SPDX-License-Identifier: MIT-0

 Permission is hereby granted, free of charge, to any person obtaining a copy of this
 software and associated documentation files (the "Software"), to deal in the Software
 without restriction, including without limitation the rights to use, copy, modify,
 merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 permit persons to whom the Software is furnished to do so.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""

"""
This class is compatible with Amazon Lex V2 format.
"""
import logging

from amazon_lex_helper import LexEvent
from amazon_lex_helper import Disambiguation
from amazon_lex_helper import LexResponse

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


class LexEventDispatcher:

    def __init__(self):
        self.subscribers = {}
        self.ambiguity_handler: Disambiguation = None

    def subscribe(self, *intents):
        for i in intents:
            target_intent = i.get_intent_name().lower()
            assert target_intent not in self.subscribers,\
                "More than one subscriber for same intent: {}".format(target_intent)
            self.subscribers[target_intent] = i
        return self

    def set_ambiguity_handler (self, ambiguity_handler: Disambiguation):
        self.ambiguity_handler = ambiguity_handler
        return self

    def dispatch(self, lex_request: dict) -> LexResponse:
        logger.debug("Input request = {}".format(lex_request))
        event = LexEvent(lex_request)
        if self.ambiguity_handler:
            ambiguity = self.ambiguity_handler.check_ambiguity_limit(event)
            if ambiguity:
                return self.ambiguity_handler.handle_ambiguity (ambiguity["i1"], ambiguity["i2"], ambiguity["amb"])
        intent_name = event.get_current_intent_name().lower()
        if intent_name not in self.subscribers:
            logger.debug("Warning: no observer defined for intent {}, using default behaviour".format(intent_name))
            response = LexResponse.delegate(event)
        else:
            response = self.subscribers[intent_name].process_request(event)
        logger.debug("Output response = {}".format(response))
        return response
